@@ -54,6 +54,7 @@ def index_actions():
         'reindex',
         'replicas',
         'rollover',
+        'shrink',
         'snapshot',
     ]
 
@@ -101,14 +102,21 @@ def structural_filter_elements():
         Optional('aliases'): Any(str, [str], unicode, [unicode]),
         Optional('allocation_type'): Any(str, unicode),
         Optional('count'): Coerce(int),
+        Optional('date_from'): Any(str, unicode, None),
+        Optional('date_from_format'): Any(str, unicode, None),
+        Optional('date_to'): Any(str, unicode, None),
+        Optional('date_to_format'): Any(str, unicode, None),
         Optional('direction'): Any(str, unicode),
         Optional('disk_space'): float,
         Optional('epoch'): Any(Coerce(int), None),
         Optional('exclude'): Any(int, str, unicode, bool, None),
         Optional('field'): Any(str, unicode, None),
+        Optional('intersect'): Any(int, str, unicode, bool, None),
         Optional('key'): Any(str, unicode),
         Optional('kind'): Any(str, unicode),
         Optional('max_num_segments'): Coerce(int),
+        Optional('pattern'): Any(str, unicode),
+        Optional('period_type'): Any(str, unicode),
         Optional('reverse'): Any(int, str, unicode, bool, None),
         Optional('range_from'): Coerce(int),
         Optional('range_to'): Coerce(int),
@@ -116,8 +124,10 @@ def structural_filter_elements():
         Optional('state'): Any(str, unicode),
         Optional('stats_result'): Any(str, unicode, None),
         Optional('timestring'): Any(str, unicode, None),
+        Optional('threshold_behavior'): Any(str, unicode),
         Optional('unit'): Any(str, unicode),
         Optional('unit_count'): Coerce(int),
+        Optional('unit_count_pattern'): Any(str, unicode),
         Optional('use_age'): Boolean(),
         Optional('value'): Any(int, float, str, unicode, bool),
         Optional('week_starts_on'): Any(str, unicode, None),
